@@ -23,7 +23,7 @@
 
 use warnings;
 
-my $Id = '$Id: flexml.pl,v 1.61 2006/09/13 16:34:33 wdowling Exp $ ';
+my $Id = '$Id: flexml.pl,v 1.62 2007/10/11 10:00:14 mquinson Exp $ ';
 $Id =~ s/\s*\$\s*//go;
 
 # IMPORTS.
@@ -95,6 +95,7 @@ my @tags = ();		# Tags of the DTD.
 my %roottags = ();	# Tags that may be the root tag.
 my %ctag = ();		# C variable name of each tag.
 
+my %allstates = ();     # all existing states are key of this hash
 my %states = ();	# $states{tag} is list of states used by tag element.
 my %emptytrans = ();	# $emptytrans{state} contains empty transitions in automaton.
 
@@ -425,8 +426,10 @@ sub analysechildren { # Analyse DTD children specification; return
                       # true if it may be empty.  Uses global $statecounter.
 
   my ($tag,$re,$in,$out) = @_;
+  $allstates{$in} = 1;
+  $allstates{$out} = 1;
 
-  #print "analysechildren [ $tag, $re, $in, $out ] \n";
+  print "analysechildren [ $tag, $re, $in, $out ] \n" if $debug;
 
   local $_ = $re;
 
@@ -1056,6 +1059,13 @@ if(length($tagprefix)) {
 	    $$h{$k} =~ s/>S_/>S_${tagprefix}/g;
 	}
     }
+    my %tmp = ();
+    foreach $k (keys %allstates) {
+	$k =~ s/^E_/E_${tagprefix}/;
+	$k =~ s/^S_/S_${tagprefix}/;
+	$tmp{$k} = 1;
+    }
+    %allstates = %tmp;
 }
 
 # Debugging: show DTD representation.
@@ -1369,6 +1379,24 @@ if ($scanner or $standalone) {
 	  $exitswitch .= "  }\n";
 	}
 
+	# Misplaced start or empty tag
+	unless ($nofail) {
+	    print "\n";
+	    my %ins = ();
+	    foreach (keys %allstates) {
+		$ins{$_} = 'false';
+	    }
+	    if (exists $instates{$tag}) {
+		for (split /,/,$instates{$tag}) { $ins{$_} = 'true'; }
+	    }
+	    $ins{"ROOT_${tagprefix}$myctag"} = $roottags{$tag} ? 'true' : 'false';
+	    my $str = '<' . join(',', grep {$ins{$_} eq 'false'} keys %ins);	    
+	    if ($str ne '<') {
+		print "$str>\"<$tag\"{s} FAIL(\"Starting tag <$tag> is not allowed here.\");\n";
+	    }
+	}
+	
+	  
 	# Start or empty tag: initialise attribute list.
 	print "\n";
 	if ($roottags{$tag}) {
